/* 
 * BRC - Beat Rate Counter, (c) Petri Honkamaa, 2002
 * 
 * Beat Rate Counter is a simple command line 
 * program to count beat rates (e.g. heart beats
 * or the measures of the music). 
 * 
 * If 'SHOW_STAT' is defined the program shows
 * also some basic statistics.
 *
 * For more documentation use "Brc -h"
 */

/* Common */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#define SHOW_STAT

/* Windows specific */
#ifdef _WIN32
#include <windows.h>
#include <conio.h>
#define PRINTF printf
void init_own() {}
void clear_own() {}

/* Linux specific */
#else
#include <sys/time.h>
#include <unistd.h>
#include <curses.h>
#define PRINTF printw
void init_own() { 
	initscr(); cbreak(); noecho(); nonl(); 
	idlok(stdscr,TRUE); scrollok(stdscr, TRUE);       
}
void clear_own() { endwin(); }
unsigned long GetTickCount() {
	struct timeval t;
	gettimeofday(&t,NULL);
	return ((1000*t.tv_sec)+(t.tv_usec/1000)); // return in ms
}
#endif

/* Functions */
void show_help()
{
	printf ("BRC - Beat Rate Counter, (c) Petri Honkamaa, 2002\n"
		"\n"
 		"Beat Rate Counter is a simple command line program to count\n"
		"beat rates (e.g. heart beats or the measures of the music).\n"
		"\n"
		"Usage is simple: Just beat [Enter] to the rate of the beats\n"
		"you want to measure. When you are satisfied with the\n"
		"result, hit any other key.\n"
		"\n"
 		"The results are given in beats per minute (bpm). Value is\n"
		"calculated by dividing the number of [Enter]-hits (after the\n"
		"first one) by the time since the first hit.\n"
#ifdef SHOW_STAT
		"\n"
		"The program also shows two simple statistical values:\n"
		"'M.D.' and 's'. Both of these are based on the differences\n"
		"of local bpm-values compared to the global average. Local\n"
		"bpm-values are based on two sequential [Enter]-hits.\n"
		"\n"
		"'M.D.' is the average of the absolute values of the\n"
		"above differences.\n"
		"\n"
		"'s' is the square root of the average of the squared\n"
		"differences.\n"
#endif
 	);
 }

/* Main */
int main(int argc, char* argv[])
{
	unsigned long start_tick;
	unsigned long tick;
	int beat_count=0;
	double bpm;
#ifdef SHOW_STAT
	int i;
	unsigned long diffs[256];
	unsigned long prev_tick=0;
	int diffs_len=0;
	double stat_md;
	double stat_s;
	double tmp;
#endif
	if (argc > 1) {
		show_help();
		return EXIT_SUCCESS;
	}

	init_own();

	PRINTF("BRC - Beat Rate Counter, (c) Petri Honkamaa, 2002\n");
	PRINTF("Start beating [Enter] when you're ready\n");
	if (getch() != 13) {
		clear_own();
		show_help();
		return EXIT_SUCCESS;
	}
	start_tick=GetTickCount();
	PRINTF("-\n");
	while (getch() == 13)
	{
		// First the result
		tick=GetTickCount()-start_tick;
		beat_count++;
		bpm=60000.0*beat_count/tick;
		PRINTF("%0.2f bpm",bpm);
		PRINTF(" (%2d in %.2fs)", 
			beat_count,
			(double)tick/1000.0);
#ifdef SHOW_STAT
		// Then the statistics that are based on 
		// two sequential [Enter]-hits
		if (diffs_len >= 254) break;
		diffs[diffs_len++]=tick-prev_tick; 
		prev_tick=tick;
		stat_md=0;
		for (i=0; i<diffs_len; i++) {
			tmp=(60000.0/diffs[i]);
			stat_md+=fabs(tmp-bpm);
			stat_s+=((tmp-bpm)*(tmp-bpm));
		}
		stat_md=(double)(stat_md/diffs_len);
		stat_s=sqrt((double)(stat_s/diffs_len));
		PRINTF(" \t- Locally %0.2f bpm (M.D.=%0.2f s=%0.2f)",
			60000.0/diffs[diffs_len-1],
			stat_md,
			stat_s);
#endif
		// Change the line
		PRINTF("\n");
	}
	PRINTF("Done\n");

	clear_own();
	return EXIT_SUCCESS;
}

